@@ -24,13 +24,13 @@
 # CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
-
+import binascii
 import codecs
 import hashlib
 import re
 from binascii import unhexlify
 from math import log10
-from typing import Any, Callable, List, Optional, Tuple, Union, cast
+from typing import Any, Callable, ClassVar, Dict, Optional, Sequence, Union, cast
 
 from .._codecs import _pdfdoc_encoding_rev
 from .._protocols import PdfObjectProtocol, PdfWriterProtocol
@@ -38,7 +38,6 @@ from .._utils import (
     StreamType,
     b_,
     deprecation_with_replacement,
-    hexencode,
     logger_warning,
     read_non_whitespace,
     read_until_regex,
@@ -71,7 +70,7 @@ class PdfObject(PdfObjectProtocol):
         self,
         pdf_dest: PdfWriterProtocol,
         force_duplicate: bool = False,
-        ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
+        ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "PdfObject":
         """
         clone object into pdf_dest (PdfWriterProtocol which is an interface for PdfWriter)
@@ -80,6 +79,10 @@ class PdfObject(PdfObjectProtocol):
             a new copy is always performed
         ignore_fields : list/tuple of Fields names (for dictionaries that will
             be ignored during cloning (apply also to childs duplication)
+            if fields are to be considered for a limited number of levels
+            you have to add it as integer:
+            eg  [1,"/B","/TOTO"] means "/B" will be ignored at first level only
+            but "/TOTO" on all levels
         in standard, clone function call _reference_clone (see _reference)
 
         Args:
@@ -155,7 +158,7 @@ class NullObject(PdfObject):
         self,
         pdf_dest: PdfWriterProtocol,
         force_duplicate: bool = False,
-        ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
+        ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "NullObject":
         """Clone object into pdf_dest."""
         return cast(
@@ -197,7 +200,7 @@ class BooleanObject(PdfObject):
         self,
         pdf_dest: PdfWriterProtocol,
         force_duplicate: bool = False,
-        ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
+        ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "BooleanObject":
         """Clone object into pdf_dest."""
         return cast(
@@ -257,7 +260,7 @@ class IndirectObject(PdfObject):
         self,
         pdf_dest: PdfWriterProtocol,
         force_duplicate: bool = False,
-        ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
+        ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "IndirectObject":
         """Clone object into pdf_dest."""
         if self.pdf == pdf_dest and not force_duplicate:
@@ -374,7 +377,7 @@ class FloatObject(float, PdfObject):
         self,
         pdf_dest: Any,
         force_duplicate: bool = False,
-        ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
+        ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "FloatObject":
         """Clone object into pdf_dest."""
         return cast(
@@ -421,7 +424,7 @@ class NumberObject(int, PdfObject):
         self,
         pdf_dest: Any,
         force_duplicate: bool = False,
-        ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
+        ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "NumberObject":
         """Clone object into pdf_dest."""
         return cast(
@@ -471,7 +474,7 @@ class ByteStringObject(bytes, PdfObject):
         self,
         pdf_dest: Any,
         force_duplicate: bool = False,
-        ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
+        ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "ByteStringObject":
         """Clone object into pdf_dest."""
         return cast(
@@ -495,7 +498,7 @@ class ByteStringObject(bytes, PdfObject):
 
             bytearr = RC4_encrypt(encryption_key, bytearr)  # type: ignore
         stream.write(b"<")
-        stream.write(hexencode(bytearr))
+        stream.write(binascii.hexlify(bytearr))
         stream.write(b">")
 
     def writeToStream(
@@ -505,7 +508,7 @@ class ByteStringObject(bytes, PdfObject):
         self.write_to_stream(stream, encryption_key)
 
 
-class TextStringObject(str, PdfObject):
+class TextStringObject(str, PdfObject):  # noqa: SLOT000
     """
     A string object that has been decoded into a real unicode string.
 
@@ -518,7 +521,7 @@ class TextStringObject(str, PdfObject):
         self,
         pdf_dest: Any,
         force_duplicate: bool = False,
-        ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
+        ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "TextStringObject":
         """Clone object into pdf_dest."""
         obj = TextStringObject(self)
@@ -554,9 +557,7 @@ class TextStringObject(str, PdfObject):
         else:
             raise Exception("no information about original bytes")
 
-    def write_to_stream(
-        self, stream: StreamType, encryption_key: Union[None, str, bytes] = None
-    ) -> None:
+    def get_encoded_bytes(self) -> bytes:
         # Try to write the string out as a PDFDocEncoding encoded string.  It's
         # nicer to look at in the PDF file.  Sadly, we take a performance hit
         # here for trying...
@@ -564,6 +565,12 @@ class TextStringObject(str, PdfObject):
             bytearr = encode_pdfdocencoding(self)
         except UnicodeEncodeError:
             bytearr = codecs.BOM_UTF16_BE + self.encode("utf-16be")
+        return bytearr
+
+    def write_to_stream(
+        self, stream: StreamType, encryption_key: Union[None, str, bytes] = None
+    ) -> None:
+        bytearr = self.get_encoded_bytes()
         if encryption_key:
             from .._security import RC4_encrypt
 
@@ -590,10 +597,10 @@ class TextStringObject(str, PdfObject):
         self.write_to_stream(stream, encryption_key)
 
 
-class NameObject(str, PdfObject):
+class NameObject(str, PdfObject):  # noqa: SLOT000
     delimiter_pattern = re.compile(rb"\s+|[\(\)<>\[\]{}/%]")
     surfix = b"/"
-    renumber_table = {
+    renumber_table: ClassVar[Dict[str, bytes]] = {
         "#": b"#23",
         "(": b"#28",
         ")": b"#29",
@@ -605,7 +612,7 @@ class NameObject(str, PdfObject):
         self,
         pdf_dest: Any,
         force_duplicate: bool = False,
-        ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
+        ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "NameObject":
         """Clone object into pdf_dest."""
         return cast(
